@@ -8,6 +8,7 @@
 #include "logging.h"
 #include "surfacehelper.h"
 #include "waylandxdgactivationv1_p.h"
+#include "waylandxdgdialogv1_p.h"
 #include "waylandxdgforeignv2_p.h"
 
 #include <KWaylandExtras>
@@ -16,10 +17,12 @@
 #include "qwayland-plasma-window-management.h"
 #include <QEvent>
 #include <QGuiApplication>
+#include <QLibraryInfo>
 #include <QPixmap>
 #include <QPoint>
 #include <QString>
 #include <QTimer>
+#include <QVersionNumber>
 #include <QWaylandClientExtensionTemplate>
 #include <QWindow>
 #include <qpa/qplatformnativeinterface.h>
@@ -224,16 +227,24 @@ void WindowSystem::setMainWindow(QWindow *window, const QString &handle)
     if (window->isExposed()) {
         doSetMainWindow(window, handle);
     } else {
-        // We can only import an XDG toplevel. QtWayland currently has no proper signal
-        // for shell surface creation. wlSurfaceCreated() is too early.
-        // Instead, we wait for it being exposed and then set its parent.
+        // We can only import an XDG toplevel.
+        // QWaylandWindow::surfaceRoleCreated is only in Qt 6.8,
+        // in earlier versions wait for the window be exposed,
+        // since QWaylandWindow::wlSurfaceCreated is too early.
+#if QT_VERSION < QT_VERSION_CHECK(6, 8, 0)
         window->setProperty(c_kdeXdgForeignPendingHandleProperty, handle);
         window->installEventFilter(this);
+#else
+        connect(waylandWindow, &QNativeInterface::Private::QWaylandWindow::surfaceRoleCreated, window, [window, handle] {
+            doSetMainWindow(window, handle);
+        });
+#endif
     }
 }
 
 bool WindowSystem::eventFilter(QObject *watched, QEvent *event)
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 8, 0)
     if (event->type() == QEvent::Expose) {
         auto *window = static_cast<QWindow *>(watched);
         if (window->isExposed()) {
@@ -246,6 +257,7 @@ bool WindowSystem::eventFilter(QObject *watched, QEvent *event)
             window->removeEventFilter(this);
         }
     }
+#endif
 
     return QObject::eventFilter(watched, event);
 }
@@ -275,6 +287,23 @@ void WindowSystem::doSetMainWindow(QWindow *window, const QString &handle)
     connect(imported, &QObject::destroyed, waylandWindow, [waylandWindow] {
         waylandWindow->setProperty(c_kdeXdgForeignImportedProperty, QVariant());
     });
+
+    // Before Qt 6.10, Qt sets XDG Dialog modal only when it has a transient parent.
+    if (QLibraryInfo::version() < QVersionNumber(6, 10, 0)) {
+        auto *oldDialog = waylandWindow->findChild<WaylandXdgDialogV1 *>();
+        if (window->modality() != Qt::NonModal && !oldDialog) {
+            auto &xdgDialog = WaylandXdgDialogWmV1::self();
+            if (xdgDialog.isActive()) {
+                if (auto *xdgToplevel = xdgToplevelForWindow(window)) {
+                    auto *dialog = xdgDialog.getDialog(xdgToplevel);
+                    dialog->set_modal();
+                    dialog->setParent(waylandWindow);
+                }
+            }
+        } else {
+            delete oldDialog;
+        }
+    }
 }
 
 #include "moc_windowsystem.cpp"
